uniform float zNear = 0.3;
uniform float zFar = 1500.0;

#if SHADOWQUALITY > 0
in float blockBrightness;

in vec4 shadowCoordsFar;
uniform sampler2D shadowMapFar;
uniform float shadowIntensity = 1;
uniform float shadowMapWidthInv;
uniform float shadowMapHeightInv;

#endif

#if SHADOWQUALITY > 1
in vec4 shadowCoordsNear;
uniform sampler2D shadowMapNear;
#endif

uniform float flatFogDensity;
uniform float flatFogStart;

float linearDepth(float depthSample)
{
    depthSample = 2.0 * depthSample - 1.0;
    float zLinear = 2.0 * zNear / (zFar + zNear - depthSample * (zFar - zNear));
    return zLinear;
}

vec4 applyFog(vec4 rgbaPixel, float fogWeight) {
	return vec4(mix(rgbaPixel.rgb, rgbaFog.rgb, fogWeight), rgbaPixel.a);
}

vec4 applyFogAndShadow(vec4 rgbaPixel, float fogWeight) {
	#if SHADOWQUALITY > 0
	float totalFar = 0.0;
	if (shadowCoordsFar.w > 0) {
		for (int x = -1; x <= 1; x++) {
			for (int y = -1; y <= 1; y++) {
				float nearestLight = texture(shadowMapFar, shadowCoordsFar.xy + vec2(x * shadowMapWidthInv, y * shadowMapHeightInv)).r;
				if (shadowCoordsFar.z > nearestLight + 0.0015) {
					totalFar += 1.0;
				}
			}
		}
	}

	totalFar /= 9.0;
	
	float b = 1.0 - shadowIntensity * totalFar * shadowCoordsFar.w * 0.45;
	#endif
	
	
	#if SHADOWQUALITY > 1
	float totalNear = 0.0;
	if (shadowCoordsNear.w > 0) {
		for (int x = -2; x <= 2; x++) {
			for (int y = -2; y <= 2; y++) {
				float nearestLight = texture(shadowMapNear, shadowCoordsNear.xy + vec2(x * shadowMapWidthInv, y * shadowMapHeightInv)).r;
				if (shadowCoordsNear.z > nearestLight + 0.0015 - (SHADOWQUALITY-1)*0.00006) {
					totalNear += 1.0;
				}
			}
		}
	}
	
	totalNear /= 25.0;
	b -=  shadowIntensity * totalNear * shadowCoordsNear.w * 0.45;
	#endif
	
	#if SHADOWQUALITY > 0
	b = clamp(b + blockBrightness, 0, 1);
	
	rgbaPixel *= vec4(b, b, b, 1);
	#endif
	
	return applyFog(rgbaPixel, fogWeight);
}	


float getFogLevel(float fogMin, float fogDensity, float worldPosY) {
	float depth = min(250, gl_FragCoord.z);
	
	float val = max(
		fogMin + 1 - 1 / exp(depth * fogDensity + max(flatFogDensity / (flatFogDensity < 0 ? 8 : 1) * depth * (	-5*flatFogStart) / 100, 0)),
		(1 - 1 / exp((worldPosY - flatFogStart) * flatFogDensity)) - max(0.2, (1 - depth))
	);
	
	
	return clamp(val, 0, 1);
}