uniform vec3 sunPosition;
uniform float playerToSealevelOffset;
uniform int ditherSeed;
uniform int horizontalResolution;
uniform float fogWaveCounter;
uniform sampler2D glow;
uniform sampler2D sky;

//	<https://www.shadertoy.com/view/4dS3Wd>
//	By Morgan McGuire @morgan3d, http://graphicscodex.com
//
float hash(float n) { return fract(sin(n) * 1e4); }
float hash(vec2 p) { return fract(1e4 * sin(17.0 * p.x + p.y * 0.1) * (0.1 + abs(sin(p.y * 13.0 + p.x)))); }

float noise(float x) {
	float i = floor(x);
	float f = fract(x);
	float u = f * f * (3.0 - 2.0 * f);
	return mix(hash(i), hash(i + 1.0), u);
}

float noise(vec3 x) {
	const vec3 step = vec3(110, 241, 171);

	vec3 i = floor(x);
	vec3 f = fract(x);
 
	// For performance, compute the base input to a 1D hash from the integer part of the argument and the 
	// incremental change to the 1D based on the 3D -> 1D wrapping
    float n = dot(i, step);

	vec3 u = f * f * (3.0 - 2.0 * f);
	return mix(mix(mix( hash(n + dot(step, vec3(0, 0, 0))), hash(n + dot(step, vec3(1, 0, 0))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 0))), hash(n + dot(step, vec3(1, 1, 0))), u.x), u.y),
               mix(mix( hash(n + dot(step, vec3(0, 0, 1))), hash(n + dot(step, vec3(1, 0, 1))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 1))), hash(n + dot(step, vec3(1, 1, 1))), u.x), u.y), u.z);
}


void getSkyColorAt(vec3 skyPosition, vec3 sunPosition, float sealevelOffsetFactor, float skyLightIntensity, float horizonFog, out vec4 skyColor, out vec4 skyGlow)
{
    vec3 V = normalize(skyPosition.xyz);
	vec3 V2 = normalize(vec3(skyPosition.x, skyPosition.y + sealevelOffsetFactor * playerToSealevelOffset, skyPosition.z));
	vec3 L = sunPosition.xyz;
	
    // Compute the proximity of this fragment to the sun.

    float vl = dot(V, L);
	
	// Look up the sky color and glow colors.
	
	float invHorizonDistance = 1 - V2.y;
	
	vec4 Kg = texture(glow,  vec2((L.y + 1.0) / 2.0, 1 - (vl + 1.0) / 2.0));
    vec4 Ks = texture(sky, vec2((L.y + 1.0) / 2.0, invHorizonDistance));

    // Combine the color and glow
    skyColor = vec4(Ks.rgb + Kg.rgb * Kg.a / 2.0, Ks.a);
	
	// Apply fog
	float fogAmount = max(
		fogMinIn + max(fogDensityIn * 120 - 0.12, 0) + max(-flatFogDensity * gl_FragCoord.z * flatFogStart / 5, 0), 
		(1 - 1 / exp((skyPosition.y - flatFogStart) * flatFogDensity))
	);
	
	// Add a little extra fog near the horizon
	float f = 0;
	
	float rnd = fogWaveCounter / 1.0;
	vec3 rndPos = vec3(rnd + skyPosition.x / 100.0, skyPosition.y / 100.0, skyPosition.z / 100.0 - rnd);
	
	float density = invHorizonDistance * (0.6 + noise(rndPos)/3);

	float fac = max(0, density - 0.5) * max(min(1, 5 * fogAmount), horizonFog);
	
	
	
	fogAmount += fac;
	fogAmount = clamp(fogAmount, 0, 1);
	
	
	skyColor = applyFog(skyColor, fogAmount);
	skyColor = mix(skyColor, vec4(rgbaFog.rgb, fogAmount), 1 - skyLightIntensity);
	
	skyColor += NoiseFromPixelPosition(ivec2(gl_FragCoord.xy), ditherSeed, horizontalResolution);
	
	#if GODRAYS > 0
		float intensity = clamp(V2.y/4 + (vl/2 - 0.3) , 0, 0.5);
		
		skyColor.rgb *= 1 - intensity * 2 * 0.2;
		skyGlow = vec4(0, intensity, 0, 1);
	#else
		skyGlow = vec4(0, 0, 0, 1);
	#endif
}
