uniform float windWaveCounter;
uniform float waterWaveCounter;
uniform vec3 playerpos;
uniform float globalWarpIntensity;

uniform float windWaveIntensity = 1;

// https://github.com/ashima/webgl-noise

vec4 mod289(vec4 x)
{
  return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec4 permute(vec4 x)
{
  return mod289(((x*34.0)+1.0)*x);
}

vec4 taylorInvSqrt(vec4 r)
{
  return 1.79284291400159 - 0.85373472095314 * r;
}

vec2 fade(vec2 t) {
  return t*t*t*(t*(t*6.0-15.0)+10.0);
}

// Classic Perlin noise
float cnoise(vec2 P)
{
  vec4 Pi = floor(P.xyxy) + vec4(0.0, 0.0, 1.0, 1.0);
  vec4 Pf = fract(P.xyxy) - vec4(0.0, 0.0, 1.0, 1.0);
  Pi = mod289(Pi); // To avoid truncation effects in permutation
  vec4 ix = Pi.xzxz;
  vec4 iy = Pi.yyww;
  vec4 fx = Pf.xzxz;
  vec4 fy = Pf.yyww;

  vec4 i = permute(permute(ix) + iy);

  vec4 gx = fract(i * (1.0 / 41.0)) * 2.0 - 1.0 ;
  vec4 gy = abs(gx) - 0.5 ;
  vec4 tx = floor(gx + 0.5);
  gx = gx - tx;

  vec2 g00 = vec2(gx.x,gy.x);
  vec2 g10 = vec2(gx.y,gy.y);
  vec2 g01 = vec2(gx.z,gy.z);
  vec2 g11 = vec2(gx.w,gy.w);

  vec4 norm = taylorInvSqrt(vec4(dot(g00, g00), dot(g01, g01), dot(g10, g10), dot(g11, g11)));
  g00 *= norm.x;  
  g01 *= norm.y;  
  g10 *= norm.z;  
  g11 *= norm.w;  

  float n00 = dot(g00, vec2(fx.x, fy.x));
  float n10 = dot(g10, vec2(fx.y, fy.y));
  float n01 = dot(g01, vec2(fx.z, fy.z));
  float n11 = dot(g11, vec2(fx.w, fy.w));

  vec2 fade_xy = fade(Pf.xy);
  vec2 n_x = mix(vec2(n00, n01), vec2(n10, n11), fade_xy.x);
  float n_xy = mix(n_x.x, n_x.y, fade_xy.y);
  return 2.3 * n_xy;
}


vec4 applyVertexWarping(int renderFlags, vec4 worldPos) {

	#if WAVINGSTUFF == 1

	// bit 12 is the water waving on/off flag
	if ((renderFlags & 0x1000) > 0) {
		worldPos.y += cnoise(vec2((worldPos.xz + playerpos.xz) / 3 + vec2(waterWaveCounter / 2, 0))) / 10;
	} else {
		// Coopers reed water layer has bit 11 and 12 set. Lets just ignore bit 11 in this case
		
		// bit 11 is the wind waving on/off flag
		if ((renderFlags & 0x800) > 0) {
			float x = windWaveCounter + worldPos.x / 5.0;
			worldPos.x += windWaveIntensity * ((sin(x / 2) + sin(0.5 + 2*x) + sin(1 + 3*x)/3) / 30.0);
			float z = -windWaveCounter + worldPos.z / 5.0;
			worldPos.z += windWaveIntensity * ((sin(z / 2) + sin(0.5 + 2*z) + sin(1 + 3*z)/3) / 30.0);
		}
	}
	
	if (globalWarpIntensity > 0) {
		float x = 0 + (worldPos.x + playerpos.x) / 5.0 + (worldPos.y + playerpos.y) / 8.0;
		worldPos.x += (sin(x / 2) + sin(0.5 + 2*x) + sin(1 + 3*x)/3) / 30.0 * globalWarpIntensity;
		float z = -0 + (worldPos.z + playerpos.z) / 5.0 + (worldPos.y + playerpos.y) / 10.0;
		worldPos.z += (sin(z / 2) + sin(0.5 + 2*z) + sin(1 + 3*z)/3) / 30.0 * globalWarpIntensity;
		
		//float y = -0 - (worldPos.z + playerpos.z) / 5.0;
		//worldPos.y += (sin(x / 2) + sin(0.5 + 2*z) + sin(1 + 3*z)/4) / 200.0 * globalWarpIntensity;
	}
	
	#endif
		
	return worldPos;
}




