#version 330 core

uniform sampler2D terrainTex;

uniform float blockTextureSize;
uniform float textureAtlasSize;
uniform float waterFlowCounter;

in vec4 rgba;
in vec4 rgbaFog;
in float fogAmount;
in vec2 uv;
in vec2 uvSize;
in float waterStillCounterOff;
flat in vec2 uvBase;

in vec2 flowVectorf;
in float glowLevel;

layout(location = 0) out vec4 outAccu;
layout(location = 1) out vec4 outReveal;
layout(location = 2) out vec4 outGlow;

#include fogandlight.fsh

void drawPixel(vec4 color) {
    float alpha = color.a;

	float weight = color.a * clamp(0.03 / (1e-5 + pow(gl_FragCoord.z / 200, 4.0)), 1e-2, 1e3);
	
    // RGBA32F texture (accumulation)
    outAccu = vec4(color.rgb * color.a, color.a) * weight;

    // R32F texture (revealage)
    // Make sure to use the red channel (and GL_RED target in your texture)
    outReveal.r = alpha*1.2;
	
    outGlow = vec4(glowLevel, 0, 0, color.a);
}

void main() 
{
	vec4 texColor;

	vec2 uvInverse = uvBase + blockTextureSize - uvSize;

	if (length(flowVectorf) > 0.001) {
		vec2 flowVec = normalize(flowVectorf);
		float maxsize = blockTextureSize - 0.1f / textureAtlasSize;
		
		vec2 uvxOffset = mod((uv - uvBase) + flowVec * waterFlowCounter * blockTextureSize, vec2(maxsize));
		
		texColor = texture(terrainTex, uvBase + uvxOffset);
	} else {
		texColor = texture(terrainTex, uv) * waterStillCounterOff + (1-waterStillCounterOff) * texture(terrainTex, uvInverse);				
	}
	
	
	drawPixel(applyFogAndShadow(texColor * rgba, fogAmount));	
}