#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout(location = 0) in vec3 vertexPositionIn;
layout(location = 1) in vec2 uvIn;
// rgb = block light, a=sun light level
layout(location = 2) in vec4 rgbaLightIn;
// rgb = ao, biome tint, block side shadings
layout(location = 3) in vec4 rgbaBlockIn;

// Bits 0-7: Glow level
// Bits 8-10: Z-Offset
// Bit 11: Wind waving yes/no
// Bit 12: Water waving yes/no
layout(location = 4) in int renderFlags;
layout(location = 5) in vec2 flowVector;

// Byte 0: Should wave yes/no
// Byte 1: Distance to upper left corner, where 255 = size of the block texture
// Byte 2: Distance to upper right corner, where 255 = size of the block texture
layout(location = 6) in int waterFlagsIn;

uniform float waterStillCounter;

uniform vec4 rgbaFogIn;
uniform vec3 rgbaAmbientIn;
uniform float fogDensityIn;
uniform float fogMinIn;
uniform vec3 origin;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;
uniform float blockTextureSize;


out vec2 flowVectorf;
out vec4 rgba;
out vec4 rgbaFog;
out float fogAmount;
out vec2 uv;
out vec2 uvSize;
out float waterStillCounterOff;
flat out vec2 uvBase;


#include shadowcoords.vsh
#include fogandlight.vsh
#include vertexwarp.vsh


void main(void)
{
	vec4 worldPos = vec4(vertexPositionIn + origin, 1.0);
	
	worldPos = applyVertexWarping(renderFlags | (waterFlagsIn & 1) << 12, worldPos);
		
	float x = mod(waterStillCounter + length(worldPos.xz + playerpos.xz)/3, 2);

	waterStillCounterOff = smoothstep(0, 1, abs(x - 1));
	
	
	vec4 cameraPos = modelViewMatrix * worldPos;
	
	gl_Position = projectionMatrix * cameraPos;
	rgba = applyLight(rgbaAmbientIn, rgbaLightIn, rgbaBlockIn, renderFlags, cameraPos);
	rgbaFog = rgbaFogIn;
	fogAmount = getFogLevel(worldPos, fogMinIn, fogDensityIn);
	uv = uvIn;
	
	float w= ((waterFlagsIn >> 8) & 0xff) / 255.0 * blockTextureSize;
	float h = ((waterFlagsIn >> 16) & 0xff) / 255.0 * blockTextureSize;
	uvSize = vec2(w, h);
	uvBase = uv - uvSize;
	
	
	flowVectorf = flowVector;
	
	
	
	calcShadowMapCoords(modelViewMatrix, worldPos);
	
	// We pretend the decal is closer to the camera to enforce it always being drawn on top
	// Required e.g. when water is besides stairs or slabs
	gl_Position.w += 0.0008;
}