#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

uniform sampler2D terrainTex;
uniform float alphaTest;

in vec4 rgba;
in vec4 rgbaFog;
in float fogAmount;
in vec2 uv;
in float glowLevel;
flat in int renderFlags;

layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outGlow;

#include fogandlight.fsh

void main() 
{
	vec4 texColor = texture(terrainTex, uv) * rgba;
	outColor = applyFogAndShadow(texColor, fogAmount);
	
	// The OpaqueNoCull Pass renders with alpha blending
	// This makes tall grass very neatly fade out in the distance
	// but looks quite bad on leaves, so we use the highest bit in the render flags byte
	// to make only the leaves a bit less opaque
	if ((renderFlags & 64) > 0) {
		float alphaModifier = clamp(2.5 * linearDepth(gl_FragCoord.z) - 0.2, 0, 0.5);
		outColor.a = min(1, outColor.a + alphaModifier);
	}
	
	if (outColor.a < alphaTest) discard;
	outGlow = vec4(glowLevel, 0, 0, outColor.a);
}