#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout(location = 0) in vec3 vertexPositionIn;
layout(location = 1) in vec2 uvIn;
// rgb = block light, a=sun light level
layout(location = 2) in vec4 rgbaLightIn;
// rgb = ao, biome tint, block side shadings
layout(location = 3) in vec4 rgbaBlockIn;
layout(location = 4) in int renderFlags;
layout(location = 5) in vec2 uv2In;
layout(location = 6) in vec4 rgbaBlock2In;


uniform vec4 rgbaFogIn;
uniform vec3 rgbaAmbientIn;
uniform float fogDensityIn;
uniform float fogMinIn;
uniform vec3 origin;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;

out vec4 rgba; // Without biome tint
out vec4 rgba2; // With biome tint
out vec4 rgbaFog;
out float fogAmount;
out vec2 uv;
out vec2 uv2;

#include shadowcoords.vsh
#include fogandlight.vsh
#include vertexwarp.vsh

void main(void)
{
	vec4 worldPos = vec4(vertexPositionIn + origin, 1.0);

	worldPos = applyVertexWarping(renderFlags, worldPos);
	
	vec4 cameraPos = modelViewMatrix * worldPos;

	
	gl_Position = projectionMatrix * cameraPos;
	
	calcShadowMapCoords(modelViewMatrix, worldPos);
	
	fogAmount = getFogLevel(worldPos, fogMinIn, fogDensityIn);
	uv = uvIn;
	uv2 = uv2In;
	
	rgba = applyLight(rgbaAmbientIn, rgbaLightIn, rgbaBlockIn, renderFlags, cameraPos);
	rgba2 = applyLight(rgbaAmbientIn, rgbaLightIn, rgbaBlock2In, renderFlags, cameraPos);
	rgba2.a = rgbaBlock2In.a;
	rgba.a = rgbaBlockIn.a;
	rgbaFog = rgbaFogIn;
}
