#version 330 core

uniform sampler2D primaryScene;
uniform sampler2D glowParts;  // The second color buffer (outGlow var)
uniform sampler2D bloomParts; // The blurred find bright texture
uniform sampler2D godrayParts;

uniform float gammaLevel;
uniform float brightnessLevel;
uniform float sepiaLevel;
uniform float damageVignetting;

in vec2 invFrameSize;
in vec2 texCoord;
flat in float godrayIntensity;

layout(location = 0) out vec4 outColor;

#include fxaa.fsh

vec4 ColorGrade(vec4 c) {
	// Fat Sepia
	vec3 sepia = vec3(
		(c.r * 0.393) + (c.g * 0.769) + (c.b * 0.189),
		(c.r * 0.349) + (c.g * 0.686) + (c.b * 0.168),
		(c.r * 0.272) + (c.g * 0.534) + (c.b * 0.131)
	);
	
	c.rgb = mix(c.rgb, sepia, sepiaLevel);
	
	// Neat greenish mix
	/*c.rgb = max(vec3(0.0), c.rgb * 1.1 - 0.1);
	c.r *= 7.0;
	c.r /= c.r + 2.5;
	c.b *= 0.7;*/
	
	// Gamma correction
	c.rgb = pow(c.rgb, vec3(1.0 / gammaLevel));
	
	c.rgb *= brightnessLevel;
	
	return c;
}


void main(void)
{
	// FXAA precompiler constant is set by game engine
	#if FXAA == 1
		vec4 color = fxaaTexturePixel(primaryScene, texCoord, invFrameSize);
	#else
		vec4 color = texture(primaryScene, texCoord);
	#endif

    
	color.a=1;
	#if BLOOM == 1
		vec4 bloomCol = texture(bloomParts, texCoord);
		float glowLevel = texture(glowParts, texCoord).r;
		
		//vec3 blendedBloomCol = (bloomCol.rgb + color.rgb * (1-bloomCol.a));
		//color.rgb = (color.rgb + blendedBloomCol) / 2;
		
		float ambLevel = AMBIENTBLOOMLEVEL / 200.0;
		
		color.rgb = (color.rgb + bloomCol.rgb * (0.5+ambLevel/2)) / (1 + ambLevel);

	#endif 
	
	#if GODRAYS > 0
		color += texture(godrayParts, texCoord) * godrayIntensity;
		color.a=1;
	#endif
	
	
	color = ColorGrade(color);
	
	
	
	
	// Vignetting
	vec2 position = (gl_FragCoord.xy * invFrameSize.xy) - vec2(0.5); 
	float vignette = 1 - smoothstep(1.1- damageVignetting / 4, 0.75 - 0.45, length(position));
	color.rgb = mix(color.rgb, vec3(0.8 * damageVignetting/2, 0, 0), vignette);
	
	// Limit brightness
	float b = (color.r + color.b + color.g) / 3;
	color.rgb /= max(1, b);
	
	
	outColor = color;
}