#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout (location = 0) in vec3 vertexPosition;	// Per vertex
layout (location = 1) in vec2 uv;						// Per vertex
layout (location = 2) in vec4 baseColor;			// Per vertex
layout (location = 3) in int renderFlags;	 		// Per instance
layout (location = 4) in vec3 particlePosition; // Per instance
layout (location = 5) in vec4 uvTransform;		// Per instance
layout (location = 6) in float scale;					// Per instance
layout (location = 7) in vec4 rgbaLightIn; 		// Per instance
layout (location = 8) in vec4 rgbaBlockIn; 		// Per instance

uniform vec4 rgbaFogIn;
uniform vec3 rgbaAmbientIn;	
uniform float fogMinIn;
uniform float fogDensityIn;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;

out vec4 color;
out vec4 rgbaFog;
out float fogAmount;

#include shadowcoords.vsh
#include fogandlight.vsh
#include vertexwarp.vsh

void main()
{
	mat4 mvmat = modelViewMatrix;
	
	float x = particlePosition.x + scale / 8;
	float y = particlePosition.y; ;
	float z = particlePosition.z + scale / 8;
	
	// 1. Translate the particle
	mvmat[3][0] = mvmat[0][0] * x + mvmat[1][0] *y + mvmat[2][0] * z + mvmat[3][0];
	mvmat[3][1] = mvmat[0][1] * x + mvmat[1][1] *y + mvmat[2][1] * z + mvmat[3][1];
	mvmat[3][2] = mvmat[0][2] * x + mvmat[1][2] *y + mvmat[2][2] * z + mvmat[3][2];
	mvmat[3][3] = mvmat[0][3] * x + mvmat[1][3] *y + mvmat[2][3] * z + mvmat[3][3];
	
	
	// 2. Billboard the particle
	mvmat[0][0] = 1.0;
	mvmat[0][1] = 0.0;
	mvmat[0][2] = 0.0;

	mvmat[1][0] = 0.0;
	mvmat[1][1] = 1.0;
	mvmat[1][2] = 0.0;

	mvmat[2][0] = 0.0;
	mvmat[2][1] = 0.0;
	mvmat[2][2] = 1.0;
	
	
	// 3. Lighting
	vec4 worldPos = vec4(vertexPosition * scale, 1.0);
	
	worldPos = applyVertexWarping(renderFlags, worldPos);
	
	vec4 cameraPos = mvmat * worldPos;
	color = baseColor * applyLight(rgbaAmbientIn, rgbaLightIn, rgbaBlockIn, renderFlags, cameraPos);
	color.a = rgbaBlockIn.a;
	rgbaFog = rgbaFogIn;


	// 4. Done.
	gl_Position = projectionMatrix * cameraPos;
	fogAmount = getFogLevel(cameraPos, fogMinIn, fogDensityIn);
	
	//calcShadowMapCoords(modelViewMatrix, cameraPos);
}